#include <LEDA/basic.h>
#include <LEDA/d_array.h>
#include <LEDA/graph.h>
#include <LEDA/list.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <LEDA/string.h>
#include <LEDA/integer.h>
#include <LEDA/stream.h>
// Makros, Konstanten
// ******************
#define EMPTY 0
#define COMPLETE 1

class Tgl_Graph;
class Tgl_Selection;
// OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
// OO tgl_obj - enthlt alle zur C - Ebene gehrenden OO
// OO           Implementationen, die nicht Teil des  OO
// OO           Interpreters Sind                     OO
// OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO

// DEBUG HELP
void wait(void) {
     string s;
     cin >> s;
   }

// **************************
// Seriennummer 
// **************************
//
class ID {
public:
    // @@@ value
    int num;
  /* */
  // ~~~ CONSTRUCTORS
  ID() { num=-1; }
  // ~~~ get (liefert die Seriennummer)
  int get() { return  num; }
  // ~~~ inc (erhht die Sereiennummer um 1)
  void inc() { ++num; };
};


// **************************
// Itemtypes (Nodes,Edges)
// **************************
//
// Nodes
// **************************
class Tgl_Vtype {
private:
  // help node zur Duplikaterstellung
  node help;
public:
  // @@@ Attribute
  d_array<string, string> attributes;
  /* */
  // Methoden, die auf help zugreifen ...
  friend Tgl_Graph; 
  // Konstruktor
  Tgl_Vtype() {
    help=NULL;
  }
  // OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Vtype node) {
    string i;
    int first=1;
    forall_defined(i,node.attributes){ 
      if (first!=1) { str << " , "; }
      str << i << " " << node.attributes[i];
      first=0;
    }
    str << " ; ";
    return str;
  }
  // OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Vtype &node) {
    char i[255],value[255];
    while(1){
      str >> i;
      if (strcmp(";",i)==0) {
	break;
      } else {
	str >> value;
	node.attributes[i]=value;
	str >> i;
	if (strcmp(";",i)==0) break;
      }
    }
    return str;
  }
};

//
// Edges
// **************************
class Tgl_Etype {
public:
  // @@@ Attribute
  d_array<string,string> attributes;
  /* */
  // OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Etype edge) {
    string i;
    int first=1;
    forall_defined(i,edge.attributes){ 
      if (first!=1) { str << " , "; }
      str << i << " " << edge.attributes[i];
      first=0;
    }
    str << " ; ";
    return str;
  }
   // OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Etype &edge) {
    char i[255],value[255];
    while(1){
      str >> i;
      if (strcmp(";",i)==0) {
	break;
      } else {
	str >> value;
	edge.attributes[i]=value;
	str >> i;
	if (strcmp(";",i)==0) break;
      }
    }
    return str;
  }
};

// **************************
// Selection
// **************************
class Tgl_Selection {
public:
  // @@@ Referenz auf den zugehrigen Graphen
  Tgl_Graph* of;
  // @@@ Liste der enthaltenen Knoten
  list<node> nodes;
  // @@@ Liste der enthaltenen Ecken
  list<edge> edges;
  // @@@ Seriennummer
  static ID id;
  /* */
  // *OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Selection &S);
  // *OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Selection &S);
  // *OPERATOR: =
  Tgl_Selection& operator=(const Tgl_Selection &S);
  // *OPERATOR: +
  Tgl_Selection operator+(const Tgl_Selection &S);
  // OPERATOR -
  Tgl_Selection operator-(const Tgl_Selection &S);
  // *~~~ Erzeugen (Konstruktor) a)
  Tgl_Selection();
  Tgl_Selection(Tgl_Graph* G);
  // *Kopierkonstruktor
  Tgl_Selection (const Tgl_Selection &S);
  // ~~~ Zerstren (Destruktor) b)
  ~Tgl_Selection();
  // ~~~ Knoten, Kante hinzufgen c)
  Tgl_Selection& add(const node n);
  Tgl_Selection& add(const edge e);
  Tgl_Selection& add(const Tgl_Selection &S);
  Tgl_Selection& add_node(int n);
  Tgl_Selection& add_edge(int n);
  // ~~~ Knoten, Kanten herausnehmen d)
  Tgl_Selection& remove(const node n);
  Tgl_Selection& remove(const edge e);
  Tgl_Selection& remove(const Tgl_Selection &S);
  // ~~~ Knoten, KAnten lschen e)
  Tgl_Selection& erase();
  // ~~~ Selection leeren f)
  Tgl_Selection& clear();
  // ~~~ Gre der Selection g)
  int count_nodes();
  int count_edges();
  // ~~~ Ausgabe des n. Knotens, Kante h)
  node node_nr(int n);
  edge edge_nr(int n);
   // ~~~ Setzen der Knoten, Kantenattribute i)
  int set_node_attribute(string attribute, string value);
  int set_edge_attribute(string attribute, string value);
  int set_one_node_attribute(string attribute, string value, int i=0);
  int set_one_edge_attribute(string attribute, string value, int i=0);
  // ~~~ Auslesen der Kanten, Knotenattribute j)
  string get_node_attribute(string attribute, int i=0);
  string get_edge_attribute(string attribute, int i=0);
  // ~~~ Filter k)
  Tgl_Selection& select_sources();
  Tgl_Selection& select_targets();
  Tgl_Selection& select_sources_targets();
  Tgl_Selection& select_outs();
  Tgl_Selection& select_ins();
  Tgl_Selection& select_ins_outs();
  Tgl_Selection& select_nodes();
  Tgl_Selection& select_edges();
  Tgl_Selection& select_node_attribute(string attribute, string value);
  Tgl_Selection& select_edge_attribute(string attribute, string value);
  // ~~~ Operationen l)
  Tgl_Selection& join(Tgl_Selection &S);
  Tgl_Selection& intersect(Tgl_Selection &S);
  Tgl_Selection& setminus(Tgl_Selection &S);
};
// static Initialisierung
ID Tgl_Selection::id;

// **************************
// Graph
// **************************
class Tgl_Graph {
public:
  // @@@ Name des Graphen
  string name;
  // @@@ Graph
  GRAPH<Tgl_Vtype,Tgl_Etype> leda_G;
  // @@@ Alle definierten Selections
  list<Tgl_Selection*> selections;
  // Muster eines Knotens
  Tgl_Vtype generic_node;
  // Muster einer Kante
  Tgl_Etype generic_edge;
  // Liste aller definierten Knotenattribute
  list<string> node_attributes;
  // Liste aller definierten Kantenattribute
  list<string> edge_attributes;
  // @@@ Seriennummer
  static ID id;
public:
  /* */
  // OPERATOR: <<
  friend ostream &operator <<(ostream& str, Tgl_Graph& G);
  // OPERATOR: >> 
  friend istream &operator >>(istream& str, Tgl_Graph& G);
  // ~~~ Erzeugen (Konstruktor) a)
  Tgl_Graph();
  // Kopierkonstruktor
  Tgl_Graph(const Tgl_Graph& G);
  // ~~~ Zerstren (Destruktor) b)
  ~Tgl_Graph();
  // ~~~ Graph leeren d)
  void clear();
  // ~~~ Graphen Laden e)
  int read(string file);
  // ~~~ Graphen Speichern f)
  int write(string file);
  // ~~~ Knoten einfgen g)
  Tgl_Selection* new_node();
  // ~~~ Kante einfgen h)
  Tgl_Selection* new_edge(Tgl_Selection& from, Tgl_Selection& to);
  // ~~~ Anzahl der Knoten, Kanten i)
  int count_edges();
  int count_nodes();
  // ~~~ Definition einers Knoten, Kantenattributes j)
  void new_node_attribute(string attribute, string generic_value);
  void new_edge_attribute(string attribute, string generic_value);
  // ~~~ Lschen eines Knoten, Kantenattributes k) 
  int delete_node_attribute(string attribute);
  int delete_edge_attribute(string attribute);
  // ~~~ Anzahl der Knoten, KAntenattribute l)
  int count_node_attributes();
  int count_edge_attributes();
  // ~~~ Ausgabe des n. Knoten, Kantenattributes m)
  string get_node_attribute(int n);
  string get_edge_attribute(int n);
  // ~~~ Operationen n)
  void join(Tgl_Graph& G);
  // ~~~ Hinzufgen einer Selection
  void paste(Tgl_Selection& S, Tgl_Selection& E);
};
// static Initialisierung
ID Tgl_Graph::id;

// ...................................
// ooooooooooooooooooooooooooooooooooo
// OOOOOOOOO IMPLEMENTATION OOOOOOOOOO
// ooooooooooooooooooooooooooooooooooo
// '''''''''''''''''''''''''''''''''''
// compare
int compare(Tgl_Selection* const &A, Tgl_Selection* const &B){
  if (A==B) { return 0;} else { return -60;}
}

// **************************
// Selection (IMPLEMENTATION)
// **************************
  // *** OPERATOR: <<
  ostream& operator <<(ostream &str, Tgl_Selection &S) {
    node n=NULL;
    edge e=NULL;
    str << "Tgl_Selection(" << &S << ")" << endl;
    str << "[of]" << endl;
    str << S.of->name << endl;
    str << "[nodes]" << endl;
    forall(n,S.nodes) {
      str << (S.nodes.rank(n)-1) << ") ";
      str << S.of->leda_G.inf(n) << endl; 
    }
    if (n==NULL) { str << "empty;" << endl; }
    str << "[edges]" << endl;
    forall(e,S.edges) {
      str << (S.edges.rank(e)-1) << ") ";
      str << S.of->leda_G.inf(e) << endl;
    }
    if (e==NULL) { str << "empty;" << endl; }
    str << endl;
    return str;  
  }
  // *** OPERATOR: >>
  istream& operator >>(istream &str, Tgl_Selection &S) {
    return str;
  }
  // OPERATOR: =
  Tgl_Selection& Tgl_Selection::operator=(const Tgl_Selection &S){
    of=S.of;
    nodes=S.nodes;
    edges=S.edges;
    return*this;
  }
  // OPERATOR +
  Tgl_Selection Tgl_Selection::operator+(const Tgl_Selection &S){
    Tgl_Selection Sresult(*this); 
    Sresult.add(S);
    return Sresult;
  }
  // OPERATOR -
  Tgl_Selection Tgl_Selection::operator-(const Tgl_Selection &S){
    Tgl_Selection Sresult(*this); 
    Sresult.remove(S);
    return Sresult;
  }
  // *** Erzeugen (Konstruktor) a)
  Tgl_Selection::Tgl_Selection(){
    cout << "ERROR(tgl_obj):undefined KONSTRUKTOR Tgl_Selection() used!" << endl;
  }
  Tgl_Selection::Tgl_Selection(Tgl_Graph* G){
    of=G;
    G->selections.push(this);
    id.inc();
  }
  Tgl_Selection::Tgl_Selection(const Tgl_Selection& S) :
  of(S.of), nodes(), edges() {
    of=S.of;
    of->selections.push(this);
    nodes=S.nodes;
    edges=S.edges;
    id.inc();
  }
  // ~~~ Zerstren (Destruktor) b)
  Tgl_Selection::~Tgl_Selection() {
    // Eintrag im Zugehrigen Graphen lschen
    list_item it=of->selections.search(this);
    if (it!=NULL) {
      of->selections.del_item(it);
    }
  }
  // ~~~ Knoten, Kante hinzufgen c)
  Tgl_Selection& Tgl_Selection::add(const node n) {
    if (nodes.search(n) == NULL) {
      nodes.push(n);
    }
    return *this;
  }
  Tgl_Selection& Tgl_Selection::add(const edge e) {
    if (edges.search(e) == NULL) {
      edges.push(e);
    }
    return *this;
  }

  Tgl_Selection& Tgl_Selection::add(const Tgl_Selection &S) {
    node n;
    edge e;
    forall(n,S.nodes) {
      this->add(n);
    }
    forall(e,S.edges) {
      this->add(e);
    }
    return *this;
  }

  Tgl_Selection& Tgl_Selection::add_node(int n) {
    if (n<(of->count_nodes())) {
      list<node> L=of->leda_G.all_nodes();
      this->add(L.inf(L.item(n)));
    } else { 
      cout << "ERROR(tgl_obj): index exceeded in add_node(int n)!" << endl;
    }
    return *this;
  }
  Tgl_Selection& Tgl_Selection::add_edge(int n) {
    if (n<(of->count_edges())) {
      list<edge> L=of->leda_G.all_edges();
      this->add(L.inf(L.item(n)));
    } else { 
      cout << "ERROR(tgl_obj): index exceeded in add_edge(int n)!" << endl;
    }
    return *this;
  }

  // ~~~ Knoten, Kanten herausnehmen d)
  Tgl_Selection& Tgl_Selection::remove(const node n) {
    list_item it=this->nodes.search(n);
    if (it != NULL) {
      this->nodes.del_item(it);
    }
    return *this;
  }

  Tgl_Selection& Tgl_Selection::remove(const edge e) {
    list_item it=this->edges.search(e);
    if (it != NULL) {
      this->edges.del_item(it);
    }
    return *this;
  }

  Tgl_Selection& Tgl_Selection::remove(const Tgl_Selection &S) {
    /* Selection aus sich selbst entfernen geht nicht! */
    if (&S==this) { 
      cout << "ERROR(Tgl_obj): remove(Tgl_Selection)";
      cout << "called rekursively" << endl;
      return *this;
    }
    node n;
    edge e;
    forall(n,S.nodes) {
      this->remove(n);
    }
    forall(e,S.edges) {
      this->remove(e);
    }
    return *this;
  }

  // ~~~ Knoten, KAnten lschen e)
  Tgl_Selection& Tgl_Selection::erase() {
    node n;
    edge e, ee;
    Tgl_Selection* S;
    list<edge> Ledges;
    /* Kanten aus dem Graphen entfernen */
    forall(e,edges) {
      of->leda_G.del_edge(e);
    }
    /* Knoten aus dem Graphen entfernen */
    /* in_out edges der Knoten in Ledges zusammenfhren */
    /* und in Sinoutedges sammeln */
    forall(n,nodes) {
      forall_inout_edges(ee,n) {
	this->add(ee);
      }
      of->leda_G.del_node(n);
    }
    /* Knoten, Kanten aus allen anderen(!) Selections entfernen */
    forall(S,of->selections) {
      /* Rekursion vermeiden! */
      if (S!=this) { S->remove(*this); } 
    }
    this->clear();
    return *this;
  }
  // ~~~ Selection leeren f)
  Tgl_Selection& Tgl_Selection::clear() {
    this->nodes.clear();
    this->edges.clear();
    return *this;
  }
  // ~~~ Gre der Selection g)
  int Tgl_Selection::count_nodes() {
    return this->nodes.size();
  }
  int Tgl_Selection::count_edges() {
    return this->edges.size();
  }
  // ~~~ Ausgabe des n. Knotens, Kante h)
  node Tgl_Selection::node_nr(int n) {
    if (n<this->count_nodes()) {
      return this->nodes.inf(this->nodes.item(n));
    } else {
      cout << "ERROR(tgl_obj): index exceeded in node_nr(int n)!" << endl;
      return NULL;
    }
  }
  edge Tgl_Selection::edge_nr(int n) {
    if (n<this->count_edges()) {
      return this->edges.inf(this->edges.item(n));
    } else {
      cout << "ERROR(tgl_obj): index exceeded in edge_nr(int n)!" << endl;
      return NULL;
    }
  }
   // ~~~ Setzen der Knoten, Kantenattribute i)
  int Tgl_Selection::set_node_attribute(string attribute, string value) {
    node n;
    if (!(this->of->generic_node.attributes.defined(attribute))) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return 1;
    } else { 
      forall(n,this->nodes) {
	this->of->leda_G[n].attributes[attribute]=value;

      }
      return 0;
    }
  }
  int Tgl_Selection::set_edge_attribute(string attribute, string value) {
    edge e;
    if (!(this->of->generic_edge.attributes.defined(attribute))) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return 1;
    } else { 
      forall(e,this->edges) {
	this->of->leda_G[e].attributes[attribute]=value;
      }
      return 0;
    }
  }
   // ~~~ Setzen eines bestimmten Knoten, Kantenattributes i)
  int Tgl_Selection::set_one_node_attribute(string attribute, string value,
					       int i) {
    if ( (!(this->of->generic_node.attributes.defined(attribute))) ||
    i>=this->count_nodes()) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return 1;
    } else {
      node n=this->nodes.inf(this->nodes.item(i));
      this->of->leda_G[n].attributes[attribute]=value;
      return 0;
    }
  }
  int Tgl_Selection::set_one_edge_attribute(string attribute, string value,
					   int i) {
    if ( (!(this->of->generic_edge.attributes.defined(attribute))) ||
    i>=this->count_edges()) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return 1;
    } else {
      edge e=this->edges.inf(this->edges.item(i));
      this->of->leda_G[e].attributes[attribute]=value;
      return 0;
    }
  }

  // ~~~ Auslesen der Kanten, Knotenattribute j)
  string Tgl_Selection::get_node_attribute(string attribute, int i) {
    if ( (!(this->of->generic_node.attributes.defined(attribute))) ||
    i>=this->count_nodes()) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return "";
    } else {
      node n=this->nodes.inf(this->nodes.item(i));
      return this->of->leda_G[n].attributes[attribute];
    }
  }
  string Tgl_Selection::get_edge_attribute(string attribute, int i) {
    if ( (!(this->of->generic_edge.attributes.defined(attribute))) ||
    i>=this->count_edges()) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return "";
    } else {
      edge e=this->edges.inf(this->edges.item(i));
      return this->of->leda_G[e].attributes[attribute];
    }
  }
  // ~~~ Filter k)
  Tgl_Selection& Tgl_Selection::select_sources(){
    list<edge> alledges=of->leda_G.all_edges();
    edge e;
    this->nodes.clear();
    forall(e,this->edges) {
      if (alledges.search(e)!=NULL) {
	this->add(of->leda_G.source(e));
      }
    }
    this->edges.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_targets() {
    list<edge> alledges=of->leda_G.all_edges();
    edge e;
    this->nodes.clear();
    forall(e,this->edges) {
      if (alledges.search(e)!=NULL) {
	this->add(of->leda_G.target(e));
      }
    }
    this->edges.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_sources_targets() {
    list<edge> alledges=of->leda_G.all_edges();
    edge e;
    this->nodes.clear();
    forall(e,this->edges) {
      if (alledges.search(e)!=NULL) {
	this->add(of->leda_G.target(e));
	this->add(of->leda_G.source(e));
      }
    }
    this->edges.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_outs() {
    list<node> allnodes=of->leda_G.all_nodes();
    node n;
    edge e;
    this->edges.clear();
    forall(n,this->nodes) {
      if (allnodes.search(n)!=NULL) {
	forall_out_edges(e,n) {
	  this->add(e);
	}
      }
    }
    this->nodes.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_ins() {
    list<node> allnodes=of->leda_G.all_nodes();
    node n;
    edge e;
    this->edges.clear();
    forall(n,this->nodes) {
      if (allnodes.search(n)!=NULL) {
	forall_in_edges(e,n) {
	  this->add(e);
	}
      }
    }
    this->nodes.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_ins_outs() {
    list<node> allnodes=of->leda_G.all_nodes();
    node n;
    edge e;
    this->edges.clear();
    forall(n,this->nodes) {
      if (allnodes.search(n)!=NULL) {
	forall_out_edges(e,n) {
	  this->add(e);
	}
	forall_in_edges(e,n) {
	  this->add(e);
	}
      }
    }
    this->nodes.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_nodes() {
    this->edges.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_edges() {
    this->nodes.clear();
    return *this;    
  }

  Tgl_Selection& Tgl_Selection::select_node_attribute(string attribute,
  string value) {
    node n;
    // Hilfsselection wird bentigt!
    Tgl_Selection S(*this);
      if (!(this->of->generic_node.attributes.defined(attribute))) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return *this;
    }
    forall(n,S.nodes){
      if (S.of->leda_G[n].attributes[attribute]!=value) {
	this->remove(n);
      }
    }
    this->edges.clear();
    return *this;
  }

  Tgl_Selection& Tgl_Selection::select_edge_attribute(string attribute,
  string value) {
    edge e;
    // Hilfsselection wird bentigt!
    Tgl_Selection S(*this);
      if (!(this->of->generic_edge.attributes.defined(attribute))) {
      cout << "ERROR(tgl_obj): Attribute not defined" << endl;
      return *this;
    }
    forall(e,S.edges){
      if (S.of->leda_G[e].attributes[attribute]!=value) {
	this->remove(e);
      }
    }
    this->nodes.clear();
    // Hilfsselection wird bentigt!
    this->id.num=this->id.num-1;
    return *this;
  }

  // ~~~ Operationen l)
Tgl_Selection& Tgl_Selection::join(Tgl_Selection &S){
  return this->add(S);
}

Tgl_Selection& Tgl_Selection::intersect(Tgl_Selection &S){
  node n;
  edge e;
  // Hilfsselection wird bentigt!
  Tgl_Selection X(*this);
  this->clear();

  forall(n, S.nodes) {
    if (X.nodes.search(n)!=NULL) {
      this->add(n);
    }
  }
  forall(e, S.edges) {
    if (X.edges.search(e)!=NULL) {
      this->add(e);
    }
  }
  // um die Tgl_Selection id niedrig zu halten
  this->id.num=this->id.num-1;
  return *this;
}

Tgl_Selection& Tgl_Selection::setminus(Tgl_Selection &S) { 
  this->remove(S);
  return *this;
}

     
// **************************
// Graph (IMPLEMENTATIION)
// **************************
  // *** OPERATOR: <<
  ostream& operator <<(ostream &str, Tgl_Graph &G) {
    str << "Tgl_Graph V1.0:" << endl;
    str << "[name]" << endl;
    str << G.name << endl;
    str << "[node_attributes]" << endl;
    str << G.node_attributes << " ;" << endl;
    str << "[edge_attributes]" << endl;
    str << G.edge_attributes << " ;" << endl;
    str << "[generic_node]" << endl;
    str << G.generic_node << endl;
    str << "[generic_edge]" << endl;
    str << G.generic_edge << endl;     
    str << "[leda_G]" << endl;
    G.leda_G.write_gml(str);
    return str;
  }
  // *** Erzeugen (Konstruktor) a)
  Tgl_Graph::Tgl_Graph() {
    id.inc();
    integer help(id.get());
    name="Graph"+help.tostring();
  }
  // Kopierkonstruktor
  Tgl_Graph::Tgl_Graph(const Tgl_Graph& G) : 
  leda_G(G.leda_G), selections(), generic_node(G.generic_node),
  generic_edge(G.generic_edge), node_attributes(G.node_attributes),
  edge_attributes(G.edge_attributes), name() {
    id.inc();
    this->name="copyof_"+G.name;
  }
  // *** Zerstren (Destruktor) b)
  Tgl_Graph::~Tgl_Graph() {
    // Dynamisch erzeugte Komponenten Zerstren
    list<Tgl_Selection*> X;
    X=this->selections;
    Tgl_Selection* S;
    forall(S, X) {
      delete S;
    }
  }
  // ~~~ Graph leeren d)
  void Tgl_Graph::clear() {
    // Selections des Graphen zerstren
    Tgl_Selection *S;
    forall(S,selections) {
      delete S;
    }
    // Selectionsliste leeren
    selections.clear();
    // den leda Graphen leeren
    leda_G.clear();
  }
  // ~~~ Graphen Laden e)
  int Tgl_Graph::read(string file) {
    // Tgl_ file componente lesen
    file_istream str(file+".g");
    char i[255];
    str >> i;
    if (strcmp("Tgl_Graph",i)!=0) {
      cout << "ERROR(tgl_obj): corrupt filetype in line 1!" << endl;
      return 1;
    } else {
      str >> i;
      if (strcmp("V1.0:",i)!=0) {
	cout << "ERROR(tgl_obj): not supported filetype-version!" << endl;
	return 1;
      } else {
	str >> i;
	if (strcmp("[name]",i)!=0) {
	  cout << "ERROR(tgl_obj): corrupt filetype in line 2!" << endl;
	  return 1;
	} else {
	  str >> this->name;
	}
	str >> i;
	if (strcmp("[node_attributes]",i)!=0) {
	  cout << "ERROR(tgl_obj): corrupt filetype in line 4!" << endl;
	  return 1;
	} else {
	  this->node_attributes.read(str,';');
	}
	str >> i;
	if (strcmp("[edge_attributes]",i)!=0) {
	  cout << "ERROR(tgl_obj): corrupt filetype in line 6!" << endl;
	  return 1;
	} else {
	  this->edge_attributes.read(str,';');
	}
	str >> i;
	if (strcmp("[generic_node]",i)!=0) {
	  cout << "ERROR(tgl_obj): corrupt filetype in line 8!" << endl;
	  return 1;
	} else {
	  str >> this->generic_node;
	}
	str >> i;
	if (strcmp("[generic_edge]",i)!=0) {
	  cout << "ERROR(tgl_obj): corrupt filetype in line 10!" << endl;
	  return 1;
	} else {
	  str >> this->generic_edge;
	}
      }
    }
    // LEDA-Komponente laden gml
    this->leda_G.read_gml(file+".gml");
    return 0;    
  }

  // ~~~ Graphen Speichern f)
  int Tgl_Graph::write(string file) {
    // Tgl file componente speichern
    file_ostream out(file+".g");
    out << "Tgl_Graph V1.0:" << endl;
    out << "[name]" << endl;
    out << this->name << endl;
    out << "[node_attributes]" << endl;
    out << this->node_attributes << " ;" << endl;
    out << "[edge_attributes]" << endl;
    out << this->edge_attributes << " ;" << endl;
    out << "[generic_node]" << endl;
    out << this->generic_node << endl;
    out << "[generic_edge]" << endl;
    out << this->generic_edge << endl;
    // Leda gml componente speichern
    this->leda_G.write_gml(file+".gml");
    return 0;
  }
  // ~~~ Knoten einfgen g)
  Tgl_Selection* Tgl_Graph::new_node() {
    Tgl_Selection* S = new Tgl_Selection(this);
    S->nodes.push(leda_G.new_node(generic_node));
    return S;
  }
  // ~~~ Kante einfgen h)
  Tgl_Selection* Tgl_Graph::new_edge(Tgl_Selection& from, Tgl_Selection& to) {
    Tgl_Selection* S = new Tgl_Selection(this);
    node nfrom,nto;
    forall(nfrom,from.nodes) {
      forall(nto,to.nodes) {
	S->edges.push(leda_G.new_edge(nfrom, nto, generic_edge));
      }
    }	     
      return S;
  }
  // ~~~ Anzahl der Knoten, Kanten i)
  int Tgl_Graph::count_nodes() {
    return leda_G.number_of_nodes();
  }
  int Tgl_Graph::count_edges() {
    return leda_G.number_of_edges();
  }
  // ~~~ Definition eines Knoten, Kantenattributes j)
  void Tgl_Graph::new_node_attribute(string attribute, string generic_value) {
    node n;
    this->generic_node.attributes[attribute]=generic_value;
    forall_nodes(n,this->leda_G) {
      this->leda_G[n].attributes[attribute]=generic_value;
    }
    if (this->node_attributes.search(attribute)==NULL) {
      this->node_attributes.push(attribute);
    }
  }
  void Tgl_Graph::new_edge_attribute(string attribute, string generic_value){
    edge e;
    this->generic_edge.attributes[attribute]=generic_value;
    forall_edges(e,this->leda_G) {
      this->leda_G[e].attributes[attribute]=generic_value;
    }
    if (this->edge_attributes.search(attribute)==NULL) {
      this->edge_attributes.push(attribute);
    }
  }
  // ~~~ Lschen eines Knoten, Kantenattributes k) 
  int Tgl_Graph::delete_node_attribute(string attribute) {
    list_item it=this->node_attributes.search(attribute);
    node n;
    if (it==NULL) {
      return 1;
    } else { 
      this->node_attributes.del_item(it); 
      this->generic_node.attributes.undefine(attribute);
      forall_nodes(n,this->leda_G) {
	this->leda_G[n].attributes.undefine(attribute);
      }
      return 0;
    }
  }

int Tgl_Graph::delete_edge_attribute(string attribute) {
  list_item it=this->edge_attributes.search(attribute);
  edge e;
  if (it==NULL) {
    return 1;
  } else { 
    this->edge_attributes.del_item(it); 
    this->generic_edge.attributes.undefine(attribute);
    forall_edges(e,this->leda_G) {
      this->leda_G[e].attributes.undefine(attribute);
    }
    return 0;
  }
}
// ~~~ Anzahl der Knoten, KAntenattribute l)
int Tgl_Graph::count_node_attributes() {
    return this->node_attributes.size();
  }
int Tgl_Graph::count_edge_attributes() {
    return this->edge_attributes.size();
  }
// ~~~ Ausgabe des n. Knoten, Kantenattributes m)
string Tgl_Graph::get_node_attribute(int n) {
    if (n<this->count_node_attributes()) {
      return this->node_attributes.inf(this->node_attributes.item(n));
    } else { return ""; }
  }

string Tgl_Graph::get_edge_attribute(int n){
  if (n<this->count_edge_attributes()) {
    return this->edge_attributes.inf(this->edge_attributes.item(n));
  } else { return ""; }
}

// ~~~ join mit einem anderen Graphen
void Tgl_Graph::join(Tgl_Graph& G){
  Tgl_Selection Sall(this);
  Tgl_Selection Serg(&G);
  for(int i=0;i<this->count_nodes();i++) {
    Sall.add_node(i);
  }
  for(i=0;i<this->count_edges();i++) {
    Sall.add_edge(i);
  }
  G.paste(Sall,Serg);
}

// ~~~ Pasten einer Selection
void Tgl_Graph::paste(Tgl_Selection& S, Tgl_Selection& E) {
  node n,from,to;
  edge e;
  string att;
  Tgl_Selection* NewNode;
  Tgl_Selection* NewEdge;
  Tgl_Selection From(this);
  Tgl_Selection To(this);
  E.clear();
  E.of=this;
  // neue Attribute im Graphen definieren?
  forall(att,S.of->node_attributes){
    if (this->node_attributes.search(att)==NULL) {
      this->new_node_attribute(att, S.of->generic_node.attributes[att]);
    }
  }
  forall(att,S.of->edge_attributes){
    if (this->edge_attributes.search(att)==NULL) {
      this->new_edge_attribute(att, S.of->generic_edge.attributes[att]);
    }    
  }
  // Knoten hinzufgen
  forall(n,S.nodes){
    NewNode = this->new_node();
    // Attributvalues bernehmen
    forall(att,S.of->node_attributes){
      NewNode->set_node_attribute(att,S.of->leda_G[n].attributes[att]);
    }
    // Help Komponente auf Kopie setzen
    S.of->leda_G[n].help=NewNode->node_nr(0);
    // NewNode zur Ergebnisselection hinzufgen und dann zerstren.
    E.add(*NewNode);
    delete NewNode;
  }
  // Kanten hinzufgen
  forall(e,S.edges){
    from=S.of->leda_G.source(e);
    to  =S.of->leda_G.target(e);
    // Startknoten schon vorhanden?
    if (S.of->leda_G[from].help==NULL) {
      // Startknoten erzeugen
      NewNode = this->new_node();
      // Attributvalues bernehmen
      forall(att,S.of->node_attributes){
	NewNode->set_node_attribute(att,S.of->leda_G[from].attributes[att]);
      }
      // Help Komponente auf Kopie setzen
      S.of->leda_G[from].help=NewNode->node_nr(0);
      // NewNode zur Ergebnisselection hinzufgen dann zerstren.
      E.add(*NewNode);
      delete NewNode;
    }
    // Zielknoten schon vorhanden?
    if (S.of->leda_G[to].help==NULL) {
      // Zielknoten erzeugen
      NewNode = this->new_node();
      // Attributvalues bernehmen
      forall(att,S.of->node_attributes){
	NewNode->set_node_attribute(att,S.of->leda_G[to].attributes[att]);
      }
      // Help Komponente auf Kopie setzen
      S.of->leda_G[to].help=NewNode->node_nr(0);
      // NewNode zur Ergebnisselection hinzufgen und dann zerstren.
      E.add(*NewNode);
      delete NewNode;
    }
    // Kante erzeugen
    From.clear(); From.add(S.of->leda_G[from].help);
    To.clear()  ; To.add(S.of->leda_G[to].help);
    NewEdge=this->new_edge(From,To);
    // Attributvalues bernehmen
    forall(att,S.of->edge_attributes){
      NewEdge->set_edge_attribute(att,S.of->leda_G[e].attributes[att]);
    }
    // NewEdge zur Ergebnisselection hinzufgen und dann zerstren
    E.add(*NewEdge);
    delete NewEdge;
  }
}
