#include <leda\basic.h>
#include <leda\d_array.h>
#include <leda\GRAPH.h>
#include <leda\list.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <LEDA\string.h>
#include <LEDA\integer.h>
#include <LEDA\stream.h>
// Makros, Konstanten
// ******************
#define EMPTY 0
#define COMPLETE 1

class Tgl_Graph;
class Tgl_Selection;

// **************************
// Seriennummer
// **************************
//
class ID {
public:
    // @@@ value
    int num;
  /* */
  // ~~~ CONSTRUCTORS
  ID();
  // ~~~ get (liefert die Seriennummer)
  int get();
  // ~~~ inc (erhht die Sereiennummer um 1)
  void inc();
};


// **************************
// Itemtypes (Nodes,Edges)
// **************************
//
// Nodes
// **************************
class Tgl_Vtype {
private:
  // help node zur Duplikaterstellung
  node help;
public:
  // @@@ Attribute
  d_array<string, string> attributes;
  /* */
  // Methoden, die auf help zugreifen ...
  friend Tgl_Graph;
  // Konstruktor
  Tgl_Vtype();
  // OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Vtype node);
  // OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Vtype &node);
};

//
// Edges
// **************************
class Tgl_Etype {
public:
  // @@@ Attribute
  d_array<string,string> attributes;
  /* */
  // OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Etype edge);
   // OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Etype &edge);
};

// **************************
// Selection
// **************************
class Tgl_Selection {
public:
  // @@@ Referenz auf den zugehrigen Graphen
  Tgl_Graph* of;
  // @@@ Liste der enthaltenen Knoten
  list<node> nodes;
  // @@@ Liste der enthaltenen Ecken
  list<edge> edges;
  // @@@ Seriennummer
  static ID id;
  /* */
  // *OPERATOR: <<
  friend ostream &operator <<(ostream &str, Tgl_Selection &S);
  // *OPERATOR: >>
  friend istream &operator >>(istream &str, Tgl_Selection &S);
  // *OPERATOR: =
  Tgl_Selection& operator=(const Tgl_Selection &S);
  // *OPERATOR: +
  Tgl_Selection operator+(const Tgl_Selection &S);
  // OPERATOR -
  Tgl_Selection operator-(const Tgl_Selection &S);
  // *~~~ Erzeugen (Konstruktor) a)
  Tgl_Selection();
  Tgl_Selection(Tgl_Graph* G);
  // *Kopierkonstruktor
  Tgl_Selection (const Tgl_Selection &S);
  // ~~~ Zerstren (Destruktor) b)
  ~Tgl_Selection();
  // ~~~ Knoten, Kante hinzufgen c)
  Tgl_Selection& add(const node n);
  Tgl_Selection& add(const edge e);
  Tgl_Selection& add(const Tgl_Selection &S);
  Tgl_Selection& add_node(int n);
  Tgl_Selection& add_edge(int n);
  // ~~~ Knoten, Kanten herausnehmen d)
  Tgl_Selection& remove(const node n);
  Tgl_Selection& remove(const edge e);
  Tgl_Selection& remove(const Tgl_Selection &S);
  // ~~~ Knoten, KAnten lschen e)
  Tgl_Selection& erase();
  // ~~~ Selection leeren f)
  Tgl_Selection& clear();
  // ~~~ Gre der Selection g)
  int count_nodes();
  int count_edges();
  // ~~~ Ausgabe des n. Knotens, Kante h)
  node node_nr(int n);
  edge edge_nr(int n);
   // ~~~ Setzen der Knoten, Kantenattribute i)
  int set_node_attribute(string attribute, string value);
  int set_edge_attribute(string attribute, string value);
  int set_one_node_attribute(string attribute, string value, int i=0);
  int set_one_edge_attribute(string attribute, string value, int i=0);
  // ~~~ Auslesen der Kanten, Knotenattribute j)
  string get_node_attribute(string attribute, int i=0);
  string get_edge_attribute(string attribute, int i=0);
  // ~~~ Filter k)
  Tgl_Selection& select_sources();
  Tgl_Selection& select_targets();
  Tgl_Selection& select_sources_targets();
  Tgl_Selection& select_outs();
  Tgl_Selection& select_ins();
  Tgl_Selection& select_ins_outs();
  Tgl_Selection& select_nodes();
  Tgl_Selection& select_edges();
  Tgl_Selection& select_node_attribute(string attribute, string value);
  Tgl_Selection& select_edge_attribute(string attribute, string value);
  // ~~~ Operationen l)
  Tgl_Selection& join(Tgl_Selection &S);
  Tgl_Selection& intersect(Tgl_Selection &S);
  Tgl_Selection& setminus(Tgl_Selection &S);
};
// static Initialisierung
ID Tgl_Selection::id;

// **************************
// Graph
// **************************
class Tgl_Graph {
public:
  // @@@ Name des Graphen
  string name;
  // @@@ Graph
  GRAPH<Tgl_Vtype,Tgl_Etype> leda_G;
  // @@@ Alle definierten Selections
  list<Tgl_Selection*> selections;
  // Muster eines Knotens
  Tgl_Vtype generic_node;
  // Muster einer Kante
  Tgl_Etype generic_edge;
  // Liste aller definierten Knotenattribute
  list<string> node_attributes;
  // Liste aller definierten Kantenattribute
  list<string> edge_attributes;
  // @@@ Seriennummer
  static ID id;
public:
  /* */
  // OPERATOR: <<
  friend ostream &operator <<(ostream& str, Tgl_Graph& G);
  // OPERATOR: >>
  friend istream &operator >>(istream& str, Tgl_Graph& G);
  // ~~~ Erzeugen (Konstruktor) a)
  Tgl_Graph();
  // Kopierkonstruktor
  Tgl_Graph(const Tgl_Graph& G);
  // ~~~ Zerstren (Destruktor) b)
  ~Tgl_Graph();
  // ~~~ Graph leeren d)
  void clear();
  // ~~~ Graphen Laden e)
  int read(string file);
  // ~~~ Graphen Speichern f)
  int write(string file);
  // ~~~ Knoten einfgen g)
  Tgl_Selection* new_node();
  // ~~~ Kante einfgen h)
  Tgl_Selection* new_edge(Tgl_Selection& from, Tgl_Selection& to);
  // ~~~ Anzahl der Knoten, Kanten i)
  int count_edges();
  int count_nodes();
  // ~~~ Definition einers Knoten, Kantenattributes j)
  void new_node_attribute(string attribute, string generic_value);
  void new_edge_attribute(string attribute, string generic_value);
  // ~~~ Lschen eines Knoten, Kantenattributes k)
  int delete_node_attribute(string attribute);
  int delete_edge_attribute(string attribute);
  // ~~~ Anzahl der Knoten, KAntenattribute l)
  int count_node_attributes();
  int count_edge_attributes();
  // ~~~ Ausgabe des n. Knoten, Kantenattributes m)
  string get_node_attribute(int n);
  string get_edge_attribute(int n);
  // ~~~ Operationen n)
  void join(Tgl_Graph& G);
  // ~~~ Hinzufgen einer Selection
  void paste(Tgl_Selection& S, Tgl_Selection& E);
};

